@@ -3,7 +3,7 @@
 # Entry point for linter.
 #
 # See LICENCE.md for Copyright information
-""" Init module for linter """
+"""Init module for linter."""
 
 import cmakeast.printer
 
